;;****************************************************************************
;;                      bridge.asm          bridge.asm
;;****************************************************************************
;;
;;  Copyright (C) 1989 Northwestern University, Vance Morrison
;;
;;
;; Permission to view, compile, and modify for LOCAL (intra-organization) 
;; USE ONLY is hereby granted, provided that this copyright and permission 
;; notice appear on all copies.  Any other use by permission only.
;;
;; Northwestern University makes no representations about the suitability 
;; of this software for any purpose.  It is provided "as is" without expressed 
;; or implied warranty.  See the copywrite notice file for complete details.
;;
;;*************************************************************************
;;
;; bridge.asm implemtents a simple two interface ethernet bridge
;;
;; AUTHOR: Vance Morrison
;; DATE:   5/24/89
;; ADDRESS: morrison@accuvax.nwu.edu
;;****************************************************************************

    dosseg
    .model small
    .stack 500h  
    JUMPS                       ;; deal with long displacement
                                ;; jumps automaticly

    include debug.inc
    include memory.inc
    include wd8003e.inc
    include if.inc
    include bridge.inc
    include buffer.inc
    include queue.inc
    include qif.inc
    include slip.inc
    include i8250.inc
    include packet.inc
    include 3c507.inc

       .data
    db  'Copywrite (c) 1989, Vance Morrison, Northwestern University', 13,10, 0
    db  'VERSION 1.20 ', 0
    db  'REVISION 10  ', 0

        ; a place to put the command line
comline_count db 0 
comline       db 127 dup (0)        

baud_rate_div dw 0                      ;; used for remote bridging

;; small helper macro
MY_SET_MTU MACRO name, mtu
    if_&name&_mtu = mtu
ENDM

     cur_if = 1

;;*************************************************************************
;; start declaration section.   Comment in your coices

        ;; the first WD8013EBT card
 ;   WDE_DECLARE %cur_if, 280H, 0D000H, 0H, 1, 64, 1  ;; promiscuous ,16K, 16bit
 ;   IF_DECLARE %cur_if, %cur_if, WDE
 ;   cur_if = cur_if + 1

        ;; the second WD8013EBT card
 ;   WDE_DECLARE %cur_if, 2A0H, 0D800H, 0H, 1, 64, 1  ;; promiscuous ,16K, 16bit
 ;   IF_DECLARE %cur_if, %cur_if, WDE
 ;   cur_if = cur_if + 1

        ;; the third WD8013EBT card
 ;   WDE_DECLARE %cur_if, 2C0H, 0A000H, 0H, 1, 64, 1  ;; promiscuous ,16K, 16bit
 ;   IF_DECLARE %cur_if, %cur_if, WDE
 ;   cur_if = cur_if + 1

        ;; the first WD8003EBT card
 ;   WDE_DECLARE %cur_if, 280H, 0D000H, 0H, 1, 128    ;; promiscuous,  32K
 ;   IF_DECLARE %cur_if, %cur_if, WDE
 ;   cur_if = cur_if + 1
    
        ;; the second WD8003EBT card
 ;   WDE_DECLARE %cur_if, 2A0H, 0D800H, 0H, 1, 128    ;; promiscuous,  32K
 ;   IF_DECLARE %cur_if, %cur_if, WDE
 ;   cur_if = cur_if + 1
    
        ;; the first WD8003E Ethernet card or WD8003S(H) Starlan card
     WDE_DECLARE %cur_if, 280H, 0D000H, 0H, 1, 32     ;; promiscuous,  8K
     IF_DECLARE %cur_if, %cur_if, WDE
     cur_if = cur_if + 1

        ;; a second WD8003E Ethernet card or WD8003S(H) Starlan card
 ;   WDE_DECLARE %cur_if, 2A0H, 0D800H, 0H, 1, 32     ;; promiscuous,  8K
 ;   IF_DECLARE %cur_if, %cur_if, WDE
 ;   cur_if = cur_if + 1

        ;; a serial card for a remote bridge
     I8250_DECLARE %cur_if, SLIP, %cur_if, IBM_COM1_PORT, IBM_COM1_IRQ, 0
     tmp = cur_if*10                                 ;; generate temp names
     BUFF_DECLARE %(tmp+1), 3200                      ;; the read buffer
     QUEUE_DECLARE %(tmp+2), 5, %(size qif_entry)     ;; the read queue
     BUFF_DECLARE %(tmp+3), 10240                     ;; the write buffer
     QUEUE_DECLARE %(tmp+4), 16, %(size qif_entry) ;; the write queue
     SLIP_DECLARE %cur_if,%cur_if,%(tmp+1),%(tmp+2),%(tmp+3),%(tmp+4)
     MY_SET_MTU %cur_if, 1518                         ;; overide default MTU
     Q_IF_DECLARE %cur_if,%(tmp+1),%(tmp+2),%(tmp+3),%(tmp+4),SLIP,%cur_if
     IF_DECLARE %cur_if, %cur_if, Q
     cur_if = cur_if + 1

        ;; a packet driver entry
 ;   PKT_DECLARE %cur_if, 60H, 01, 1    ;; <name> <Int num> <Class (Eth=1), prom
 ;   IF_DECLARE %cur_if, %cur_if, PKT
 ;   cur_if = cur_if + 1

        ;; a second packet driver entry
 ;   PKT_DECLARE %cur_if, 61H, 01, 1    ;; <name> <Int num> <Class (Eth=1), prom
 ;   IF_DECLARE %cur_if, %cur_if, PKT
 ;   cur_if = cur_if + 1

        ;; a 3C507 card entry 64K of memory at D0000H
 ;   C507_DECLARE %cur_if, 280H, 0D000H, 10000H, 1   ;; name,io,seg,len,prom
 ;   IF_DECLARE %cur_if, %cur_if, C507
 ;   cur_if = cur_if + 1

        ;; a 3C507 card entry 64K of memory at C0000H
 ;   C507_DECLARE %cur_if, 2A0H, 0C000H, 10000H, 1   ;; name,io,seg,len,prom
 ;   IF_DECLARE %cur_if, %cur_if, C507
 ;   cur_if = cur_if + 1

;; end declaration section
;;*************************************************************************

     num_dls = cur_if - 1
     mybridge = 1
     BDG_DECLARE %mybridge, 1, %num_dls, 1


;;************************************************************************
;; This routine converts the ASCII base ten number pointed to by SI and
;; returns the number in AX.  It also updates SI to point to the next
;; char in the string.  This routine skips preceeding space and stops
;; conversion at the first non-digit.  Clearly this routine can' handle
;; numbers up to 64K

ASCII_TO_BINARY_in_SI_out_AX_SI_const_BP_DI_ES MACRO
    local top_loop, next_char, conv_string, space_loop, done

    space_loop:
        mov CL, [SI]
        cmp CL, ' '         ; skip space
        jnz conv_string
        inc SI
        jmp space_loop
    conv_string:

    xor AX, AX
    xor CH, CH
    top_loop:
        sub CL, '0'         ; is it a digit
        jl done
        cmp CL, 9
        jg done

        mov DX, AX          ; AX = AX * 10
        shl AX, 1
        shl AX, 1
        add AX, DX
        shl AX, 1
        
        add AX, CX
    next_char:
        inc SI
    mov CL, [SI]
    jmp top_loop

    done:
ENDM


;;************************************************************************
    .code

    ;; insert code for all interfaces
    IRP idx,<1,2,3,4,5,6,7,8>
       WDE_REAL_DEFINE idx     
       I8250_REAL_DEFINE idx
       PKT_REAL_DEFINE idx     
       C507_REAL_DEFINE idx     
    endm

    DB_DEFINE           

bridge_start:
    ;; save the command line
    mov SI, 80h
    mov AX, @DATA 
    mov ES, AX
    mov DI, offset comline_count
    mov CX, 40h
    rep
    movsw

    mov AX, @DATA       ;; initialize data segment
    mov DS, AX
    cld                 ;; direction forward

        ; if this is a remote bridge
    ifdef i8250_declared
        mov SI, offset comline
        ASCII_TO_BINARY_in_SI_out_AX_SI_const_BP_DI_ES      ;; get the baud rate
        cmp AX, 0
        jnz not_default
            mov AX, 19200                   ;; 19.2 is the default
        not_default:
        shr AX, 1                           ;; compute the baud rate div
        mov BX, AX
        mov AX, 57600
        xor DX, DX
        div BX
        mov baud_rate_div, AX
    endif

    IRP idx,<1,2,3,4,5,6,7,8>       ;; call definition code
    if idx le num_dls
        WDE_DEFINE idx     

        BUFF_DEFINE %(idx*10+1)
        QUEUE_DEFINE %(idx*10+2)
        BUFF_DEFINE %(idx*10+3)
        QUEUE_DEFINE %(idx*10+4)
        Q_IF_DEFINE idx     
        SLIP_DEFINE idx     
        mov BX, baud_rate_div 
        I8250_DEFINE_in_BX idx, serial_fail

        PKT_DEFINE idx, packet_fail

        C507_DEFINE_out_AX idx, card_fail
    endif
    endm

    BDG_DEFINE %mybridge

    ;; start looking for packets
    big_loop:
    IRP idx,<1,2,3,4,5,6,7,8>
    local next
    if idx le num_dls
        BDG_IF_R_ACCESS_out_BX_CX_ES idx, next
        BDG_IF_R_FREE_const_BX_CX_BP_SI_DI_ES idx    ;; throw it away
        next:
    endif
    endm
    jmp big_loop

    packet_fail:
        print_reg <error configuring ethernet card.  Code = >, AX
        jmp terminate

    packet_fail:
        print <error configuring packet driver>
        jmp terminate

    serial_fail:
        print <error configuring serial port>
    terminate:
        mov AH, 4CH
        int 21H
END bridge_start


